<?php
/*
 +-------------------------------------------------------------------------+
 | This program is free software; you can redistribute it and/or           |
 | modify it under the terms of the GNU General Public License             |
 | as published by the Free Software Foundation; either version 2          |
 | of the License, or (at your option) any later version.                  |
 |                                                                         |
 | This program is distributed in the hope that it will be useful,
 | but WITHOUT ANY WARRANTY; without even the implied warranty of          |
 | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           |
 | GNU General Public License for more details.                            |
 +-------------------------------------------------------------------------+
*/
/*******************************************************************************
    Author ......... Jean-Michel Pepin SNMP classes (PEPJ in forum.cacti.net & http://cactiusers.org/forums/)
    Modified ....... .
    Program ........ class definition for snmp network in PHP for Cacti or other PHP software
    version ........ 0.2
    Some functions are based/extract on the work of Ian Berry for Cacti
*******************************************************************************/

/* some compatibility to CACTI */
define("REGEXP_SNMP_TRIM", "(hex|counter(32|64)|gauge|gauge(32|64)|float|ipaddress|string|integer):");
define("SNMP_METHOD_PHP", 1);
define("SNMP_METHOD_BINARY", 2);
define("SNMP_METHOD_NET", "net-snmp");
define("SNMP_METHOD_UCD", "ucd-snmp");
/* end of some compatibility to CACTI */

define("SNMP_BIN_VERSION", SNMP_METHOD_NET);
/* we must use an apostrophe to escape community names under Unix in case the user uses
characters that the shell might interpret. the ucd-snmp binaries on Windows flip out when
you do this, but are perfectly happy with a quotation mark. */
if (strtoupper(substr(PHP_OS, 0, 3)) == "WIN") {
  define("SNMP_ESCAPE_CHARACTER", "\"");
  define("SNMP_BIN_GET", "snmpget.exe");
  define("SNMP_BIN_GETNEXT", "snmpgetnext.exe");
  define("SNMP_BIN_WALK", "snmpwalk.exe");
  define("SNMP_BIN_BULKWALK", "snmpbulkwalk.exe");
  define("SNMP_BIN_SET", "snmpset.exe");
}else{
  define("SNMP_ESCAPE_CHARACTER", "'");
  define("SNMP_BIN_GET", "snmpget");
  define("SNMP_BIN_GETNEXT", "snmpgetnext");
  define("SNMP_BIN_WALK", "snmpwalk");
  define("SNMP_BIN_BULKWALK", "snmpbulkwalk");
  define("SNMP_BIN_SET", "snmpset");
}
global $path_snmp_bin,$snmp_binget,$snmp_bingetnext,$snmp_binwalk
       ,$snmp_binset,$snmp_binbulkwalk,$snmpbinversion;
$path_snmp_bin="";
$snmp_binget=SNMP_BIN_GET;
$snmp_bingetnext=SNMP_BIN_GETNEXT;
$snmp_binwalk=SNMP_BIN_WALK;
$snmp_binset=SNMP_BIN_SET;
$snmp_binbulkwalk=SNMP_BIN_BULKWALK;
$snmpbinversion=SNMP_BIN_VERSION;

/* CLASS definition */
class snmpclass
{
   /*
   * Class loading function.
   */
       var $module;
       var $param=array("hostname"=>"","version" => "1", "string" => "public", "user" => "", "password" => "","auth_proto" => "MD5","priv_pass" => "","priv_proto" => "none","context" => ""
       			,"port" => 161, "timeout" => 500, "retries" =>3, "method" =>"");
       var $ini=false, $errormsg="";
       
       function Init($objsess,$path_snmpbinary="",$snmpget="",$snmpgetnext="",$snmpwalk="",$snmpset="",$snmpbulkwalk="",$snmpbin_version="")
       {
	  $error = "";
	  if ($objsess[0]=="") $error="Error hostname is empty.";
	  else {
	   $this->snmphostname($objsess[0]);
	   if (!$this->retcode) $error=$this->errormsg;
	   $this->snmpversion($objsess[1]);
	   if (!$this->retcode) $error=$this->errormsg;
	   $this->setsnmpcommunity($objsess[2]);
	   if (!$this->retcode) $error.=$this->errormsg;
           $this->snmpuserpassword($objsess[3],$objsess[4],$objsess[5],$objsess[6],$objsess[7],true);
	   $this->setsnmp3context("");
	   if (!$this->retcode) $error.=$this->errormsg;
           $this->snmpport($objsess[8],true);
	   if (!$this->retcode) $error.=$this->errormsg;
           $this->snmptimeout($objsess[9],true);
	   if (!$this->retcode) $error.=$this->errormsg;
           $this->snmpretries($objsess[10],true);
	   if (!$this->retcode) $error.=$this->errormsg;
           $this->snmpmethod($objsess[11],$path_snmpbinary,$snmpget,$snmpgetnext,$snmpwalk,$snmpset,$snmpbulkwalk,$snmpbin_version);
	   if (!$this->retcode) $error.=$this->errormsg;
	  }
	  $this->errormsg = $error;
	  $this->ini= ($error=="");
	  $this->retcode = $this->ini;
	  return $this->retcode;
       }
       function snmphostname($hostname="")
       {
       	  $this->errormsg= "";
       	  $hashostname= ($hostname!="");
       	  if (($this->ini)||$hashostname) {
       	  	if ($hashostname) $this->param["hostname"]=$hostname;
		$this->result= $this->param["hostname"];
		$toreturn=$this->result;
		$this->retcode = 1;
          } else {
	    $this->result= "";
            $this->errormsg= "Error SNMP object not initialised.";
	    $this->retcode = 0;
	    $toreturn=0;
          }
       }
       function snmpversion($newversion)
       {
       	  $this->errormsg= "";
       	  $isgood= ($newversion!="")&& (($newversion=="1")||($newversion=="2")||($newversion=="2c")||($newversion=="3"));
       	  if (($this->ini)||$isgood) {
       	  	if ($isgood) {
       	  	   $this->param["version"]= $newversion;
       	  	   if (($newversion!="3")&& $this->param["string"]=="") $this->param["string"]="public";
       	  	}
		$this->result= $this->param["version"];
		$toreturn=$this->result;
		$this->retcode = 1;
          } else {
	    $this->result= "";
            $this->errormsg=($newversion!="")? "Error wrong SNMP version." :"Error SNMP object not initialised.";
	    $this->retcode = 0;
	    $toreturn=0;
          }
          return $toreturn;
       }
       function setsnmpcommunity($community="")
       {
       	  $this->errormsg= "";
       	  if ($community!="") $this->param["string"]=$community;
       	  //else $this->param["string"]="public";
	  $this->result= $this->param["string"];
	  $this->retcode = 1;
          return $this->result;
       }
       function snmptimeout($timeout="",$ini=false)
       {
       	  $this->errormsg= "";
       	  $isset= false;
       	  $isgood= true;
       	  if ($ini && ($timeout=="")) $timeout=500;
       	  if (is_string($timeout)) {
       	    if ($timeout!="") {
       	    	$timeout=intval($timeout);
       	    	$isset=true;
       	    	$isgood= ($timeout>0);
       	    }
       	  } else $isgood= (is_int($timeout)&&($timeout>0));
       	  if (($this->ini||$ini)&&$isgood) {
       	  	if ($isset||$ini) $this->param["timeout"]=$timeout;
		$this->result= $this->param["timeout"];
		$toreturn=$this->result;
		$this->retcode = 1;
          } else {
	    $this->result= "";
            $this->errormsg= (!$isgood)? "Error wrong SNMP parameter." :"Error SNMP object not initialised.";;
	    $this->retcode = 0;
	    $toreturn=0;
          }
          return $toreturn;
       }
       function snmpport($port="",$ini=false)
       {
       	  $this->errormsg= "";
       	  $isset= false;
       	  $isgood= true;
       	  if ($ini && ($port=="")) $port=161;
       	  if (is_string($port)) {
       	    if ($port!="") {
       	    	$port=intval($port);
       	    	$isset=true;
       	    	$isgood= ($port>0);
       	    }
       	  } else $isgood= (is_int($port)&&($port>0));
       	  if (($this->ini||$ini)&&$isgood) {
       	  	if ($isset||$ini) $this->param["port"]=$port;
		$this->result= $this->param["port"];
		$toreturn=$this->result;
		$this->retcode = 1;
          } else {
	    $this->result= "";
            $this->errormsg= (!$isgood)? "Error wrong SNMP parameter." :"Error SNMP object not initialised.";
	    $this->retcode = 0;
	    $toreturn=0;
          }
          return $toreturn;
       }
       function snmpuserpassword($user="",$password="",$auth_proto="MD5",$priv_pass="",$priv_proto="none",$initial=false)
       {
          $this->errormsg= "";
       	  $hasuser= ($user!="");
       	  if ($this->ini || $initial) {
       	    if (($this->param["version"]=="3") && $hasuser) {
       	  	if ($hasuser) $this->param["user"]=$user;
       	  	if ($hasuser) $this->param["password"]=$password;
       	  	if ($hasuser) $this->param["auth_proto"]=$auth_proto;
       	  	if ($hasuser) $this->param["priv_pass"]=$priv_pass;
       	  	if ($hasuser) $this->param["priv_proto"]=$priv_proto;
		$this->result= array("user"=> $this->param["user"]
		                     ,"password" => $this->param["password"]
		                     ,"auth_proto" => $this->param["auth_proto"]
		                     ,"priv_pass" => $this->param["priv_pass"]
		                     ,"priv_proto" => $this->param["priv_proto"]);
		$toreturn=$this->result;
		$this->retcode = 1;
	    } else {
	      if (!$initial) {
	        $this->result= array();
                $this->errormsg= (!$hasuser)? "Error SNMP user is empty or the snmp version is not 3." :"Error SNMP object not initialised.";
	        $this->retcode = 0;
	        $toreturn=0;
	      } else {
	        $this->result= array();
                $this->errormsg= "";
	        $this->retcode = 1;
	        $toreturn=1;
	      }
	    }
          } else {
	    $this->result= array();
            $this->errormsg= (!$hasuser)? "Error SNMP user is empty." :"Error SNMP object not initialised.";
	    $this->retcode = 0;
	    $toreturn=0;
          }
          return $toreturn;
       }
       function setsnmp3context($context="")
       {
       	  $this->errormsg= "";
       	  $this->param["context"]=$context;
	  $this->result= $this->param["context"];
	  $this->retcode = 1;
          return $this->result;
       }
       function getsnmp3context()
       {
       	  $this->errormsg= "";
	  $this->result= $this->param["context"];
	  $this->retcode = 1;
          return $this->result;
       }
       function snmpretries($retries="",$ini=false)
       {
       	  $this->errormsg= "";
       	  $isset= false;
       	  $isgood= true;
       	  if ($ini && ($retries=="")) $retries=3;
       	  if (is_string($retries)) {
       	    if ($retries!="") {
       	    	$retries=intval($retries);
       	    	$isset=true;
       	    	$isgood= ($retries>0);
       	    }
       	  } else $isgood= (is_int($retries)&&($retries>0));
       	  if (($this->ini||$ini)&&$isgood) {
       	  	if ($isset||$ini) $this->param["retries"]=$retries;
		$this->result= $this->param["retries"];
		$toreturn=$this->result;
		$this->retcode = 1;
          } else {
	    $this->result= "";
            $this->errormsg= (!$isgood)? "Error wrong SNMP parameter." :"Error SNMP object not initialised.";
	    $this->retcode = 0;
	    $toreturn=0;
          }
          return $toreturn;
       }
       function snmpmethod($method="",$path_binary="",$get="",$getnext="",$walk="",$set="",$binversion="")
       {
       	  global $path_snmp_bin,$snmp_binget,$snmp_bingetnext,$snmp_binwalk,$snmp_binset,$snmpbinversion;
       	  $this->errormsg= "";
       	  if (($method!=SNMP_METHOD_PHP)&&($method!=SNMP_METHOD_BINARY)) $method=SNMP_METHOD_PHP;
       	  if ($path_binary!="") $path_snmp_bin= $path_binary;
       	  if ($get!="") $snmp_binget= $get;
       	  if ($getnext!="") $snmp_bingetnext= $getnext;
       	  if ($walk!="") $snmp_binwalk= $walk;
       	  if ($set!="") $snmp_binset= $set;
       	  if ($binversion!="") $snmpbinversion= $binversion;
       	  $this->param["method"]=$method;
	  $this->result= $this->param["method"];
	  $this->retcode = 1;
          return $this->result;
       }
       function close()
       {
       	$this->ini = false;
       }
       function ListFunctions()
       {
       }
       function get($mib_oid)
       {
          $this->result= "";
          $errormsg="";
          $ret= 0;
       	  if ($this->ini &&($mib_oid!="")) {
             $ret_val = class_snmp_get($this,$mib_oid);
             $this->result=$ret_val;
             if (isset($ret_val)) {
                if ( (!substr_count($ret_val, "No Such Object")) && (!substr_count($ret_val, "Error")) ) {
                   $ret =1; // the answer could be empty
                   $this->result = trim(str_replace("\"","", $ret_val));
                } else {
                  $this->result = "";
                  $errormsg=$ret_val;
                }
             } else {
               $this->result = "";
               $errormsg="Error in function get.";
             }
          } else $errormsg="Error SNMP object not initialised or no mib/oid.";
          $this->retcode = $ret;
          $this->errormsg = $errormsg;
          return $ret;
       }
       function getnext($mib_oid)
       {
          $this->result= "";
          $this->errormsg="";
          $ret= 0;
       	  if ($this->ini &&($mib_oid!="")) {
             $ret_val = class_snmp_getnext($this,$mib_oid);
             $this->result=$ret_val;
             if (isset($ret_val)) {
                if ((strlen($ret_val) > 0)
                    && (!substr_count($ret_val, "No Such Object")) && (!substr_count($ret_val, "Error"))) {
                   $this->result = trim(str_replace("\"","", $ret_val));
                   $ret =1;
                } else {
                  $this->result = "";
                  $this->errormsg=$ret_val;
                }
             } else {
               $this->result = "";
               $this->errormsg="Error in function.";
             }
          } else $this->errormsg="Error SNMP object not initialised or no mib/oid.";
          $this->retcode = $ret;
          return $ret;
       }
       function walk($mib_oid,$type)
       {
          $ret= array();
          $errormsg="";
          $ret_ok= 0;
       	  if ($this->ini &&($mib_oid!="")) {
             $ret = class_snmp_walk($this,$mib_oid,$type);
             if (sizeof($ret)<=0) $errormsg="Error no answer: device not reached or unknow mib.";
             else $ret_ok=1;
          } else {
            $errormsg="Error SNMP object not initialised or no mib/oid.";
          }
          $this->result = $ret;
          $this->errormsg= $errormsg;
          $this->retcode = $ret_ok;
          return $ret_ok;
       }
       function set($mib_oid,$type,$val)
       {
          $this->result= "";
          $this->errormsg="";
          $ret= 0;
       	  if ($this->ini &&($mib_oid!="")) {
             $ret_val = class_snmp_set($this,$mib_oid,$type,$val);
             if ($ret_val=="1") {
                   $this->result = trim(str_replace("\"","", $ret_val));
                   $ret =1;
             } else {
               $this->errormsg="Error device not reached or mib not exist or wrong value. Could not set.";
             }
          } else $this->errormsg="Error SNMP object not initialised or no mib/oid.";
          $this->retcode = $ret;
          return $ret;
       }
       function set_list($tab_oid_value)
       {
          $this->result= "";
          $this->errormsg="";
          $ret= 0;
       	  if ($this->ini &&(sizeof($tab_oid_value)>0)) {
             $ret_val = class_snmp_set_list($this,$tab_oid_value);
             if ($ret_val=="1") {
                   $this->result = trim(str_replace("\"","", $ret_val));
                   $ret =1;
                } else {
                  $this->errormsg="Error could not set at least one value.";
                }
          } else $this->errormsg="Error SNMP object not initialised or no mib/oid.";
          $this->retcode = $ret;
          return $ret;
       }
}

/* global function in order to have not to big objects */

function class_snmp_escape_string($string) {
	if (substr_count($string, SNMP_ESCAPE_CHARACTER)) {
		$string = substr_replace(SNMP_ESCAPE_CHARACTER, "\\" . SNMP_ESCAPE_CHARACTER, $string);
	}

	return SNMP_ESCAPE_CHARACTER . $string . SNMP_ESCAPE_CHARACTER;
}

function class_snmp_get($objdevice,$oid="") {
  global $path_snmp_bin, $snmp_binget,$snmpbinversion;
  $hostname= $objdevice->param["hostname"];
  $version= $objdevice->param["version"];
  $community= $objdevice->param["string"];
  $username= $objdevice->param["user"];
  $password= $objdevice->param["password"];
  $port= $objdevice->param["port"];
  $timeout= $objdevice->param["timeout"];
  $retries= $objdevice->param["retries"];
  $auth_proto= $objdevice->param["auth_proto"];
  $priv_pass= $objdevice->param["priv_pass"];
  $priv_proto= $objdevice->param["priv_proto"];
  $context= $objdevice->param["context"];
  if ( ($priv_proto == "[None]") || ($priv_proto == "none") || ($priv_proto == "") ) {
     $proto = "authNoPriv";
     $priv_proto = "";
  }else{
     $proto = "authPriv";
  }
  $path_snmp_binget = ($path_snmp_bin=="")? $snmp_binget : $path_snmp_bin."/".$snmp_binget;
  if ($objdevice->param["method"]== SNMP_METHOD_PHP) {
    /* make sure snmp* is verbose so we can see what types of data we are getting back */
    snmp_set_quick_print(0);
    if (function_exists("snmp_set_oid_output_format")) snmp_set_oid_output_format(SNMP_OID_OUTPUT_NUMERIC);
    if ($version == "1") {
        $snmp_value = @snmpget("$hostname:$port", $community, $oid, ($timeout * 1000), $retries);
      }elseif (($version == "2")||($version == "2c")) {
        $snmp_value = @snmp2_get("$hostname:$port", $community, $oid, ($timeout * 1000), $retries);
      }else{
        $snmp_value = @snmp3_get("$hostname:$port", "$username", $proto, $auth_proto, "$password", $priv_proto, "$priv_pass", "$oid", ($timeout * 1000), $retries);
      }
  }else {
    /* ucd/net snmp want the timeout in seconds */
    $timeout = ceil($timeout / 1000);
    if ($version == "1") {
      $snmp_auth = ($snmpbinversion == SNMP_METHOD_UCD)
                     ? SNMP_ESCAPE_CHARACTER . $community . SNMP_ESCAPE_CHARACTER
                     : "-c " . SNMP_ESCAPE_CHARACTER . $community . SNMP_ESCAPE_CHARACTER; /* v1/v2 - community string */
    }elseif (($version == "2")||($version == "2c")) {
      $snmp_auth = ($snmpbinversion == SNMP_METHOD_UCD) ? SNMP_ESCAPE_CHARACTER . $community . SNMP_ESCAPE_CHARACTER : "-c " . SNMP_ESCAPE_CHARACTER . $community . SNMP_ESCAPE_CHARACTER; /* v1/v2 - community string */
      $version = "2c"; /* ucd/net snmp prefers this over '2' */
    }elseif ($version == "3") {
			if (strlen($priv_pass)) {
				$priv_pass = "-X " . class_snmp_escape_string($priv_pass) . " -x " . class_snmp_escape_string($priv_proto);
			}else{
				$priv_pass = "";
			}

			if (strlen($context)) {
				$context = "-n " . class_snmp_escape_string($context);
			}else{
				$context = "";
			}

			$snmp_auth = trim("-u " . class_snmp_escape_string($username) .
				" -l " . class_snmp_escape_string($proto) .
				" -a " . class_snmp_escape_string($auth_proto) .
				" -A " . class_snmp_escape_string($password) .
				" "    . $priv_pass .
				" "    . $context); /* v3 - username/password */
    }
    /* no valid snmp version has been set, get out */
    if (empty($snmp_auth)) { return; }
        if ($snmpbinversion == SNMP_METHOD_UCD) {
            exec($path_snmp_binget . " -O vt -v$version -t $timeout -r $retries $hostname:$port $snmp_auth $oid", $snmp_value);
        }else {
            exec($path_snmp_binget . " -O fntev $snmp_auth -v $version -t $timeout -r $retries $hostname:$port $oid", $snmp_value);
        }
    }

    if (isset($snmp_value)) {
        /* fix for multi-line snmp output */
        if (is_array($snmp_value)) {
            $snmp_value = implode(" ", $snmp_value);
        } elseif (is_bool($snmp_value) && !$snmp_value) $snmp_value = "Error device not reached or mib not exist.";
    }
    /* strip out non-snmp data */
    $snmp_value = class_snmp_format_snmp_string($snmp_value);
    return $snmp_value;
}

function class_snmp_getnext($objdevice,$oid="") {
  global $path_snmp_bin, $snmp_bingetnext,$snmpbinversion;
  $hostname= $objdevice->param["hostname"];
  $version= $objdevice->param["version"];
  $community= $objdevice->param["string"];
  $username= $objdevice->param["user"];
  $password= $objdevice->param["password"];
  $port= $objdevice->param["port"];
  $timeout= $objdevice->param["timeout"];
  $retries= $objdevice->param["retries"];
  $auth_proto= $objdevice->param["auth_proto"];
  $priv_pass= $objdevice->param["priv_pass"];
  $priv_proto= $objdevice->param["priv_proto"];
  $context= $objdevice->param["context"];
  if ( ($priv_proto == "[None]") || ($priv_proto == "none") || ($priv_proto == "") ) {
     $proto = "authNoPriv";
     $priv_proto = "";
  }else{
     $proto = "authPriv";
  }

  $path_snmp_bingetnext = ($path_snmp_bin=="")? $snmp_bingetnext : $path_snmp_bin."/".$snmp_bingetnext;
  if ($objdevice->param["method"]== SNMP_METHOD_PHP) {
    /* make sure snmp* is verbose so we can see what types of data we are getting back */
    snmp_set_quick_print(0);
    if (function_exists("snmp_set_oid_output_format")) snmp_set_oid_output_format(SNMP_OID_OUTPUT_NUMERIC);
    if ($version == "1") {
        $snmp_value = @snmpgetnext("$hostname:$port", $community, $oid, ($timeout * 1000), $retries);
      }elseif (($version == "2")||($version == "2c")) {
        $snmp_value = @snmp2_getnext("$hostname:$port", $community, $oid, ($timeout * 1000), $retries);
      }else{
	$snmp_value = @snmp3_getnext("$hostname:$port", "$username", $proto, $auth_proto, "$password", $priv_proto, "$priv_pass", "$oid", ($timeout * 1000), $retries);
      }
  }else {
    /* ucd/net snmp want the timeout in seconds */
    $timeout = ceil($timeout / 1000);
    if ($version == "1") {
      $snmp_auth = ($snmpbinversion == SNMP_METHOD_UCD)
                     ? SNMP_ESCAPE_CHARACTER . $community . SNMP_ESCAPE_CHARACTER
                     : "-c " . SNMP_ESCAPE_CHARACTER . $community . SNMP_ESCAPE_CHARACTER; /* v1/v2 - community string */
    }elseif (($version == "2")||($version == "2c")) {
      $snmp_auth = ($snmpbinversion == SNMP_METHOD_UCD) ? SNMP_ESCAPE_CHARACTER . $community . SNMP_ESCAPE_CHARACTER : "-c " . SNMP_ESCAPE_CHARACTER . $community . SNMP_ESCAPE_CHARACTER; /* v1/v2 - community string */
      $version = "2c"; /* ucd/net snmp prefers this over '2' */
    }elseif ($version == "3") {
			if (strlen($priv_pass)) {
				$priv_pass = "-X " . class_snmp_escape_string($priv_pass) . " -x " . class_snmp_escape_string($priv_proto);
			}else{
				$priv_pass = "";
			}

			if (strlen($context)) {
				$context = "-n " . class_snmp_escape_string($context);
			}else{
				$context = "";
			}

			$snmp_auth = trim("-u " . class_snmp_escape_string($username) .
				" -l " . class_snmp_escape_string($proto) .
				" -a " . class_snmp_escape_string($auth_proto) .
				" -A " . class_snmp_escape_string($password) .
				" "    . $priv_pass .
				" "    . $context); /* v3 - username/password */
    }
    /* no valid snmp version has been set, get out */
    if (empty($snmp_auth)) { return; }
        if ($snmpbinversion == SNMP_METHOD_UCD) {
            exec($path_snmp_bingetnext . " -O vt -v$version -t $timeout -r $retries $hostname:$port $snmp_auth $oid", $snmp_value);
        }else {
//print "class_snmp_getnext exec(:".($path_snmp_bin."/".$snmp_bingetnext . " -O fntev $snmp_auth -v $version -t $timeout -r $retries $hostname:$port $oid").",".$snmp_value.")"."\n";
            exec($path_snmp_bingetnext . " -O fntev $snmp_auth -v $version -t $timeout -r $retries $hostname:$port $oid", $snmp_value);
        }
    }

    if (isset($snmp_value)) {
        /* fix for multi-line snmp output */
        if (is_array($snmp_value)) {
            $snmp_value = implode(" ", $snmp_value);
        } elseif (is_bool($snmp_value) && !$snmp_value) $snmp_value = "Error device not reached or mib not exist.";
    }
    /* strip out non-snmp data */
    $snmp_value = class_snmp_format_snmp_string($snmp_value);
    return $snmp_value;
}

function class_snmp_walk($objdevice,$oid="",$type="s") {
  global $path_snmp_bin, $snmpbinversion, $snmp_binbulkwalk, $snmp_binwalk;
  $hostname= $objdevice->param["hostname"];
  $version= $objdevice->param["version"];
  $community= $objdevice->param["string"];
  $username= $objdevice->param["user"];
  $password= $objdevice->param["password"];
  $port= $objdevice->param["port"];
  $timeout= $objdevice->param["timeout"];
  $retries= $objdevice->param["retries"];
  $auth_proto= $objdevice->param["auth_proto"];
  $priv_pass= $objdevice->param["priv_pass"];
  $priv_proto= $objdevice->param["priv_proto"];
  $context= $objdevice->param["context"];
  if ( ($priv_proto == "[None]") || ($priv_proto == "none") || ($priv_proto == "") ) {
     $proto = "authNoPriv";
     $priv_proto = "";
  }else{
     $proto = "authPriv";
  }

  $snmp_array = array();
  $temp_array = array();
  $path_snmpbulkwalk = ($path_snmp_bin=="")? $snmp_binbulkwalk : $path_snmp_bin."/".$snmp_binbulkwalk;
  $path_snmpwalk = ($path_snmp_bin=="")? $snmp_binwalk : $path_snmp_bin."/".$snmp_binwalk;
  if (($objdevice->param["method"]== SNMP_METHOD_PHP) &&
      (($version == 1) || (version_compare(phpversion(), "5.1") >= 0) ||
      (!file_exists($path_snmpbulkwalk)))) {
      /* make sure snmp* is verbose so we can see what types of data we are getting back */
      /* force php to return numeric oid's */
      if (function_exists("snmp_set_oid_numeric_print")) {
        snmp_set_oid_numeric_print(TRUE);
      }
      snmp_set_quick_print(0);
      if (function_exists("snmp_set_oid_output_format")) snmp_set_oid_output_format(SNMP_OID_OUTPUT_NUMERIC);
      if ($version == "1") {
        $temp_array = @snmprealwalk("$hostname:$port", $community, $oid, ($timeout * 1000), $retries);
      }elseif (($version == "2")||($version == "2c")) {
         $temp_array = @snmp2_real_walk("$hostname:$port", $community, $oid, ($timeout * 1000), $retries);
       }else{
	 $temp_array = @snmp3_real_walk("$hostname:$port", "$username", $proto, $auth_proto, "$password", $priv_proto, "$priv_pass", "$oid", ($timeout * 1000), $retries);
       }
       $o = 0;
       for (@reset($temp_array); $i = @key($temp_array); next($temp_array)) {
         $snmp_array[$o]["oid"] = ereg_replace("^\.", "", $i);
         if ($type!="s") $snmp_array[$o]["value"] = class_snmp_format_value_string($temp_array[$i]);
         else $snmp_array[$o]["value"] = class_snmp_format_snmp_string($temp_array[$i]);
         $o++;
       }
   } else {
//print "class_snmp_walk: path_snmpwalk=$path_snmpwalk, path_snmpbulkwalk=$path_snmpbulkwalk hostname=$hostname version=$version\n";
        /* ucd/net snmp want the timeout in seconds */
        $timeout = ceil($timeout / 1000);
        if ($version == "1") {
          $snmp_auth = ($snmpbinversion == SNMP_METHOD_UCD) ? SNMP_ESCAPE_CHARACTER . $community . SNMP_ESCAPE_CHARACTER : "-c " . SNMP_ESCAPE_CHARACTER . $community . SNMP_ESCAPE_CHARACTER; /* v1/v2 - community string */
        }elseif (($version == "2")||($version == "2c")) {
          $snmp_auth = ($snmpbinversion == SNMP_METHOD_UCD) ? SNMP_ESCAPE_CHARACTER . $community . SNMP_ESCAPE_CHARACTER : "-c " . SNMP_ESCAPE_CHARACTER . $community . SNMP_ESCAPE_CHARACTER; /* v1/v2 - community string */
          $version = "2c"; /* ucd/net snmp prefers this over '2' */
        }elseif ($version == "3") {
			if (strlen($priv_pass)) {
				$priv_pass = "-X " . class_snmp_escape_string($priv_pass) . " -x " . class_snmp_escape_string($priv_proto);
			}else{
				$priv_pass = "";
			}

			if (strlen($context)) {
				$context = "-n " . class_snmp_escape_string($context);
			}else{
				$context = "";
			}

			$snmp_auth = trim("-u " . class_snmp_escape_string($username) .
				" -l " . class_snmp_escape_string($proto) .
				" -a " . class_snmp_escape_string($auth_proto) .
				" -A " . class_snmp_escape_string($password) .
				" "    . $priv_pass .
				" "    . $context); /* v3 - username/password */
        }
        if ($snmpbinversion == SNMP_METHOD_UCD) {
          $temp_array = class_snmp_exec_into_array($path_snmpwalk . " -v$version -t $timeout -r $retries $hostname:$port $snmp_auth $oid");
        }else {
          if (file_exists($path_snmpbulkwalk) && ($version > 1)) {
             $temp_array = class_snmp_exec_into_array($path_snmpbulkwalk . " -O QfntUe $snmp_auth -v $version -t $timeout -r $retries -Cr50 $hostname:$port $oid");
          }else{
            $temp_array = class_snmp_exec_into_array($path_snmpwalk . " -O QfntUe $snmp_auth -v $version -t $timeout -r $retries $hostname:$port $oid");
          }
        }
        if ((sizeof($temp_array) == 0) || (substr_count($temp_array[0], "No Such Object"))) {
           return array();
        }
        for ($i=0; $i < count($temp_array); $i++) {
          $snmp_array[$i]["oid"] = trim(ereg_replace("(.*) =.*", "\\1", $temp_array[$i]));
          if ($type!="s") $snmp_array[$i]["value"] = class_snmp_format_value_string($temp_array[$i]);
          else $snmp_array[$i]["value"] = class_snmp_format_snmp_string($temp_array[$i]);
        }
   }
   return $snmp_array;
}

function class_snmp_set($objdevice,$oid,$typeval,$value) {
  global $path_snmp_bin, $snmp_binset,$snmpbinversion;
  $hostname= $objdevice->param["hostname"];
  $version= $objdevice->param["version"];
  $community= $objdevice->param["string"];
  $username= $objdevice->param["user"];
  $password= $objdevice->param["password"];
  $port= $objdevice->param["port"];
  $timeout= $objdevice->param["timeout"];
  $retries= $objdevice->param["retries"];
  $auth_proto= $objdevice->param["auth_proto"];
  $priv_pass= $objdevice->param["priv_pass"];
  $priv_proto= $objdevice->param["priv_proto"];
  $context= $objdevice->param["context"];
  if ( ($priv_proto == "[None]") || ($priv_proto == "none") || ($priv_proto == "") ) {
     $proto = "authNoPriv";
     $priv_proto = "";
  }else{
     $proto = "authPriv";
  }

  $path_snmp_binset = ($path_snmp_bin=="")? $snmp_binset : $path_snmp_bin."/".$snmp_binset;
  if ($objdevice->param["method"]== SNMP_METHOD_PHP) {
    /* make sure snmp* is verbose so we can see what types of data we are getting back */
    snmp_set_quick_print(0);
    if (function_exists("snmp_set_oid_output_format")) snmp_set_oid_output_format(SNMP_OID_OUTPUT_NUMERIC);
    if ($version == "1") {
        $snmp_value = @snmpset("$hostname:$port", $community, $oid, $typeval,$value, ($timeout * 1000), $retries);
      }elseif ( ($version == "2")|| ($version == "2c") ) {
        $snmp_value = @snmp2_set("$hostname:$port", $community, $oid, $typeval,$value, ($timeout * 1000), $retries);
      }else{
        $snmp_value = @snmp3_set("$hostname:$port", "$username", $proto, $auth_proto, "$password", $priv_proto, "$priv_pass", "$oid", $typeval,$value, ($timeout * 1000), $retries);
      }
  }else {
    /* ucd/net snmp want the timeout in seconds */
    $timeout = ceil($timeout / 1000);
    if ($version == "1") {
      $snmp_auth = ($snmpbinversion == SNMP_METHOD_UCD)
                     ? SNMP_ESCAPE_CHARACTER . $community . SNMP_ESCAPE_CHARACTER
                     : "-c " . SNMP_ESCAPE_CHARACTER . $community . SNMP_ESCAPE_CHARACTER; /* v1/v2 - community string */
    }elseif (($version == "2")||($version == "2c")) {
      $snmp_auth = ($snmpbinversion == SNMP_METHOD_UCD) ? SNMP_ESCAPE_CHARACTER . $community . SNMP_ESCAPE_CHARACTER : "-c " . SNMP_ESCAPE_CHARACTER . $community . SNMP_ESCAPE_CHARACTER; /* v1/v2 - community string */
      $version = "2c"; /* ucd/net snmp prefers this over '2' */
    }elseif ($version == "3") {
			if (strlen($priv_pass)) {
				$priv_pass = "-X " . class_snmp_escape_string($priv_pass) . " -x " . class_snmp_escape_string($priv_proto);
			}else{
				$priv_pass = "";
			}

			if (strlen($context)) {
				$context = "-n " . class_snmp_escape_string($context);
			}else{
				$context = "";
			}

			$snmp_auth = trim("-u " . class_snmp_escape_string($username) .
				" -l " . class_snmp_escape_string($proto) .
				" -a " . class_snmp_escape_string($auth_proto) .
				" -A " . class_snmp_escape_string($password) .
				" "    . $priv_pass .
				" "    . $context); /* v3 - username/password */
    }
    /* no valid snmp version has been set, get out */
    if (empty($snmp_auth)) { return; }
        if ($snmpbinversion == SNMP_METHOD_UCD) {
            exec($path_snmp_binset . " -O vt -v$version -t $timeout -r $retries $hostname:$port $snmp_auth $oid $typeval $value", $snmp_value);
        }else {
            $cmd=$path_snmp_binset . " -O fntev $snmp_auth -v $version -t $timeout -r $retries $hostname:$port \"$oid\" \"$typeval\" \"$value\"";
//print "class_snmp_set exec(".$cmd.",\$snmp_value)"."\n";
            exec($cmd, $snmp_value);
        }
    }

    /* fix for multi-line snmp output */
    if (!isset($snmp_value)) $snmp_value = "0";
    if (is_array($snmp_value)) {
       if (sizeof($snmp_value)>0) $snmp_value = "1"; else $snmp_value = "0";
    }
    return $snmp_value;
}

/* set several SNMP values */
/* from  an array like (
 		     array("oid" => $oid_of_value_1, "type" => $type_of_value_1, "value" => $value_1)
 		    ,array(...)
 		    ,array("oid" => $oid_of_value_x, "type" => $type_of_value_x, "value" => $value_x)
			);
*/
function class_snmp_set_list($objdevice,$listvaltoset)
{
  $snmpok= "1";
  foreach($listvaltoset as $value){
    $oid = $value["oid"];$oidtype = $value["type"];$snmpval = $value["value"];
    if (($oid!="")&&($oidtype!="")) {
	$snmpok = class_snmp_set($objdevice,$oid,$oidtype,$snmpval);
	if ($snmpok!="1") break;
    }
  }
  return $snmpok;
}

function class_snmp_format_value_string($string) {
	/* strip off all leading junk (the oid and stuff) */
	$string = trim(ereg_replace(".*= ?", "", $string));
	/* remove ALL quotes */
	$string = str_replace("\"", "", $string);
	$string = str_replace("'", "", $string);
	$string = str_replace(">", "", $string);
	$string = str_replace("<", "", $string);
	$string = str_replace("\\", "", $string);

	 $k = strlen($string);
	 $val =0;
	 for ($i=0; $i < $k; $i++) {
	   $val = 256*$val+ord($string[$i]);
	 }
	 $string = strval($val);
	$string = eregi_replace(REGEXP_SNMP_TRIM, "", $string);

	return trim($string);
}

function class_snmp_format_snmp_string($string) {
	/* strip off all leading junk (the oid and stuff) */
	$string = trim(ereg_replace(".*= ?", "", $string));

	/* remove ALL quotes */
	$string = str_replace("\"", "", $string);
	$string = str_replace("'", "", $string);
	$string = str_replace(">", "", $string);
	$string = str_replace("<", "", $string);
	$string = str_replace("\\", "", $string);

	 /* Remove invalid chars */
	 $k = strlen($string);
	 for ($i=0; $i < $k; $i++) {
		if ((ord($string[$i]) <= 31) || (ord($string[$i]) >= 127)) {
			$string[$i] = " ";
		}
	 }
	 $string = trim($string);

	if (preg_match("/(hex:\?)?([a-fA-F0-9]{1,2}(:|\s)){5}/", $string)) {
		$octet = "";

		/* strip of the 'hex:' */
		$string = eregi_replace("hex: ?", "", $string);

		/* split the hex on the delimiter */
		$octets = preg_split("/\s|:/", $string);

		/* loop through each octet and format it accordingly */
		for ($i=0;($i<count($octets));$i++) {
			$octet .= str_pad($octets[$i], 2, "0", STR_PAD_LEFT);

			if (($i+1) < count($octets)) {
				$octet .= ":";
			}
		}

		/* copy the final result and make it upper case */
		$string = strtoupper($octet);
	}elseif (preg_match("/Timeticks:\s\((\d+)\)\s/", $string, $matches)) {
		$string = $matches[1];
	}

	$string = eregi_replace(REGEXP_SNMP_TRIM, "", $string);

	return trim($string);
}

/* execute_into_array - executes a command and puts each line of its output into
     an array
   @arg $command_line - the command to execute
   @returns - (array) an array containing the command output */
function class_snmp_exec_into_array($command_line) {
	exec($command_line,$out,$err);

	$command_array = array();

	for($i=0; list($key, $value) = each($out); $i++) {
		$command_array[$i] = $value;
	}

	return $command_array;
}

?>